-- ----------------------------
-- 工资发放
-- ----------------------------
DROP TABLE IF EXISTS base_personnel_labour_capital;
CREATE TABLE base_personnel_labour_capital
(
    id BIGINT NOT NULL COMMENT '主键编号',
    `create_user_id` BIGINT NULL DEFAULT NULL COMMENT '创建人',
    `create_date` DATE NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id` BIGINT NULL DEFAULT NULL COMMENT '修改人',
    `modify_date` DATE NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark` INT NOT NULL COMMENT '删除标记',
    `enabled_mark` INT NOT NULL COMMENT '有效标志',
    `name` VARCHAR(100) NULL DEFAULT NULL COMMENT '工资发放名称',
    `date_of_issue` DATE NULL DEFAULT NULL COMMENT '发放年月',
    `data_row` INT NULL DEFAULT NULL COMMENT '数据开始行',
    `id_type_column` INT NULL DEFAULT NULL COMMENT '证件类型所属列',
    `id_number_column` INT NULL DEFAULT NULL COMMENT '证件号码所属列',
    `job_number_column` INT NULL DEFAULT NULL COMMENT '工号所属列',
    `personnel_name_column` INT NULL DEFAULT NULL COMMENT '名称所属列',
    `amount_to_column` INT NULL DEFAULT NULL COMMENT '合计所属列',
    `data_file` VARCHAR(200) NULL DEFAULT NULL COMMENT '数据文件',
    PRIMARY KEY (`id`)
) ENGINE=INNODB DEFAULT CHARSET=utf8mb4 COLLATE = utf8mb4_0900_ai_ci COMMENT '工资发放';


-- ----------------------------
-- 工资发放
-- ----------------------------
DROP TABLE IF EXISTS base_personnel_labour_capital_title;
CREATE TABLE base_personnel_labour_capital_title
(
    id BIGINT NOT NULL COMMENT '主键编号',
    `create_user_id` BIGINT NULL DEFAULT NULL COMMENT '创建人',
    `create_date` DATE NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id` BIGINT NULL DEFAULT NULL COMMENT '修改人',
    `modify_date` DATE NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark` INT NOT NULL COMMENT '删除标记',
    `enabled_mark` INT NOT NULL COMMENT '有效标志',
    `labour_capital_id` BIGINT NULL DEFAULT NULL COMMENT '工资发放id',
    `name` VARCHAR(100) NULL DEFAULT NULL COMMENT '表头名称',
    `row_number` INT NULL DEFAULT NULL COMMENT '所属行',
    `column_number` INT NULL DEFAULT NULL COMMENT '所属列',
    `merge_rows` INT NULL DEFAULT NULL COMMENT '合并行数',
    `merge_columns` INT NULL DEFAULT NULL COMMENT '合并列数',
    PRIMARY KEY (`id`)
) ENGINE=INNODB DEFAULT CHARSET=utf8mb4 COLLATE = utf8mb4_0900_ai_ci COMMENT '工资发放表头';


-- ----------------------------
-- 工资条数据
-- ----------------------------
DROP TABLE IF EXISTS base_personnel_labour_capital_data;
CREATE TABLE base_personnel_labour_capital_data
(
    id BIGINT NOT NULL COMMENT '主键编号',
    `create_user_id` BIGINT NULL DEFAULT NULL COMMENT '创建人',
    `create_date` DATE NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id` BIGINT NULL DEFAULT NULL COMMENT '修改人',
    `modify_date` DATE NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark` INT NOT NULL COMMENT '删除标记',
    `enabled_mark` INT NOT NULL COMMENT '有效标志',
    `labour_capital_id` BIGINT NULL DEFAULT NULL COMMENT '工资发放id',
    `id_type` VARCHAR(100) NULL DEFAULT NULL COMMENT '证件类型',
    `id_number` VARCHAR(50) NULL DEFAULT NULL COMMENT '证件号码',
    `job_number` VARCHAR(100) NULL DEFAULT NULL COMMENT '工号',
    `name` VARCHAR(100) NULL DEFAULT NULL COMMENT '名称',
    `amount_to` VARCHAR(100) NULL DEFAULT NULL COMMENT '合计',
    `extend_json` VARCHAR(3000) NULL DEFAULT NULL COMMENT '扩展信息',
    PRIMARY KEY (`id`)
) ENGINE=INNODB DEFAULT CHARSET=utf8mb4 COLLATE = utf8mb4_0900_ai_ci COMMENT '工资条数据';


-- ----------------------------
-- 教职工经历信息
-- ----------------------------
DROP TABLE IF EXISTS wf_personnel_experience;
CREATE TABLE wf_personnel_experience
(
    id BIGINT NOT NULL COMMENT '主键编号',
    `create_user_id` BIGINT NULL DEFAULT NULL COMMENT '创建人',
    `create_date` DATE NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id` BIGINT NULL DEFAULT NULL COMMENT '修改人',
    `modify_date` DATE NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark` INT NOT NULL COMMENT '删除标记',
    `enabled_mark` INT NOT NULL COMMENT '有效标志',
    `sort_code` INT NULL DEFAULT NULL COMMENT '序号',
    `company_name` VARCHAR(50) NULL DEFAULT NULL COMMENT '企业名称',
    `learning_forms` VARCHAR(100) NULL DEFAULT NULL COMMENT '学习形式',
    `post` VARCHAR(100) NULL DEFAULT NULL COMMENT '岗位',
    `start_date` DATE NULL DEFAULT NULL COMMENT '实习开始时间',
    `end_date` DATE NULL DEFAULT NULL COMMENT '实习结束时间',
    `total_duration` BIGINT NULL DEFAULT NULL COMMENT '共计时长',
    `practice_description` VARCHAR(1000) NULL DEFAULT NULL COMMENT '实践描述',
    `annex` VARCHAR(200) NULL DEFAULT NULL COMMENT '佐证材料',
    `notes` VARCHAR(500) NULL DEFAULT NULL COMMENT '备注',
    `status` INT NOT NULL COMMENT '状态（1:结束 0:未结束）',
    PRIMARY KEY (`id`)
) ENGINE=INNODB DEFAULT CHARSET=utf8mb4 COLLATE = utf8mb4_0900_ai_ci COMMENT '教职工经历信息';


-- ----------------------------
-- 学生校级干部管理
-- ----------------------------
DROP TABLE IF EXISTS base_student_school_cadre;
CREATE TABLE base_student_school_cadre
(
    id BIGINT NOT NULL COMMENT '主键编号',
    `create_user_id` BIGINT NULL DEFAULT NULL COMMENT '创建人',
    `create_date` DATE NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id` BIGINT NULL DEFAULT NULL COMMENT '修改人',
    `modify_date` DATE NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark` INT NOT NULL COMMENT '删除标记',
    `enabled_mark` INT NOT NULL COMMENT '有效标志',
    `sort_code` INT NULL DEFAULT NULL COMMENT '序号',
    `user_number` VARCHAR(30) NULL DEFAULT NULL COMMENT '学号',
    `user_id` BIGINT NULL DEFAULT NULL COMMENT '学生id',
    `class_id` BIGINT NULL DEFAULT NULL COMMENT '班级id',
    `post` VARCHAR(50) NULL DEFAULT NULL COMMENT '职务',
    `level` VARCHAR(50) NULL DEFAULT NULL COMMENT '级别',
    `org_id` BIGINT NULL DEFAULT NULL COMMENT '学生任职机构',
    `start_time` DATE NULL DEFAULT NULL COMMENT '任职开始时间',
    `end_time` DATE NULL DEFAULT NULL COMMENT '任职结束时间',
    `status` INT NOT NULL COMMENT '状态（1:在职 0:离职）',
    `leave_reason` VARCHAR(500) NULL DEFAULT NULL COMMENT '离职原因',
    PRIMARY KEY (`id`)
) ENGINE=INNODB DEFAULT CHARSET=utf8mb4 COLLATE = utf8mb4_0900_ai_ci COMMENT '学生校级干部管理';

-- ----------------------------
-- 学生班级干部管理
-- ----------------------------
DROP TABLE IF EXISTS base_student_class_cadre;
CREATE TABLE base_student_class_cadre
(
    id BIGINT NOT NULL COMMENT '主键编号',
    `create_user_id` BIGINT NULL DEFAULT NULL COMMENT '创建人',
    `create_date` DATE NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id` BIGINT NULL DEFAULT NULL COMMENT '修改人',
    `modify_date` DATE NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark` INT NOT NULL COMMENT '删除标记',
    `enabled_mark` INT NOT NULL COMMENT '有效标志',
    `sort_code` INT NULL DEFAULT NULL COMMENT '序号',
    `user_number` VARCHAR(30) NULL DEFAULT NULL COMMENT '学号',
    `user_id` BIGINT NULL DEFAULT NULL COMMENT '学生id',
    `class_id` BIGINT NULL DEFAULT NULL COMMENT '班级id',
    `mobile` VARCHAR(30) NULL DEFAULT NULL COMMENT '联系电话',
    `post` VARCHAR(50) NULL DEFAULT NULL COMMENT '职务',
    `post_type` VARCHAR(50) NULL DEFAULT NULL COMMENT '职务类型',
    `org_id` BIGINT NULL DEFAULT NULL COMMENT '学生任职机构',
    `start_time` DATE NULL DEFAULT NULL COMMENT '任职开始时间',
    `end_time` DATE NULL DEFAULT NULL COMMENT '任职结束时间',
    `status` INT NOT NULL COMMENT '状态（1:在职 0:离职）',
    `leave_time` DATE NULL DEFAULT NULL COMMENT '离职生效日期',
    `leave_reason` VARCHAR(500) NULL DEFAULT NULL COMMENT '离职原因',
    PRIMARY KEY (`id`)
) ENGINE=INNODB DEFAULT CHARSET=utf8mb4 COLLATE = utf8mb4_0900_ai_ci COMMENT '学生班级干部管理';


-- ----------------------------
-- 快捷菜单
-- ----------------------------
DROP TABLE IF EXISTS xjr_menu_quick;
CREATE TABLE xjr_menu_quick
(
    id BIGINT NOT NULL COMMENT '主键编号',
    `create_user_id` BIGINT NULL DEFAULT NULL COMMENT '创建人',
    `create_date` DATE NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id` BIGINT NULL DEFAULT NULL COMMENT '修改人',
    `modify_date` DATE NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark` INT NOT NULL COMMENT '删除标记',
    `enabled_mark` INT NOT NULL COMMENT '有效标志',
    `sort_code` INT NULL DEFAULT NULL COMMENT '序号',
    `menu_id` BIGINT NULL DEFAULT NULL COMMENT '菜单编号',
    `user_id` BIGINT NULL DEFAULT NULL COMMENT '用户编号',
    `role_id` BIGINT NULL DEFAULT NULL COMMENT '角色编号',
    `system_id` BIGINT NULL DEFAULT NULL COMMENT '系统编号',
    PRIMARY KEY (`id`)
) ENGINE=INNODB DEFAULT CHARSET=utf8mb4 COLLATE = utf8mb4_0900_ai_ci COMMENT '快捷菜单';

