-- ------------------------------------------------
-- 资产管理盘点
-- ------------------------------------------------
DROP TABLE IF EXISTS wf_asset_manage_inventory;
CREATE TABLE `wf_asset_manage_inventory` (
  `id` BIGINT NOT NULL,
  `create_user_id` BIGINT DEFAULT NULL,
  `create_date` DATETIME(3) DEFAULT NULL,
  `modify_user_id` BIGINT DEFAULT NULL,
  `modify_date` DATETIME(3) DEFAULT NULL,
  `delete_mark` INT DEFAULT NULL,
  `enabled_mark` INT DEFAULT NULL,
  `status` SMALLINT DEFAULT 0 COMMENT '状态(0:未结束 1:结束)',
  `area_type` VARCHAR(50) DEFAULT NULL COMMENT '区域类型',
  `folder_id` BIGINT DEFAULT NULL COMMENT '附件id',
  `base_office_build_id` BIGINT DEFAULT NULL COMMENT '楼栋id',
  `floor_number` INT DEFAULT NULL COMMENT '楼层',
  `room_number` VARCHAR(50) DEFAULT NULL COMMENT '门牌号',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=INNODB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_0900_ai_ci COMMENT='资产管理盘点';


-- -----------------------------------------------------------
-- 学籍表中增加考号
-- -----------------------------------------------------------
DROP PROCEDURE IF EXISTS createCom;
DELIMITER $$
CREATE
    PROCEDURE createCom()
BEGIN
    IF NOT EXISTS(SELECT 1 FROM information_schema.columns  WHERE table_name='wf_asset_manage' AND COLUMN_NAME = 'extend_json') THEN
ALTER TABLE `wf_asset_manage`   
  ADD COLUMN `extend_json` LONGTEXT NULL   COMMENT '附件其他信息' AFTER `status`;
    END IF;
END$$
DELIMITER ;
CALL createCom;
DROP PROCEDURE createCom;

-- -----------------------------------------------------------
-- 公务派车
-- -----------------------------------------------------------
DROP TABLE IF EXISTS wf_official_vehicles;
CREATE TABLE `wf_official_vehicles` (
  `id` BIGINT NOT NULL,
  `create_user_id` BIGINT DEFAULT NULL,
  `create_date` DATETIME(3) DEFAULT NULL,
  `modify_user_id` BIGINT DEFAULT NULL,
  `modify_date` DATETIME(3) DEFAULT NULL,
  `delete_mark` INT DEFAULT NULL,
  `enabled_mark` INT DEFAULT NULL,
  `status` SMALLINT DEFAULT '0' COMMENT '状态(0:未结束 1:结束)',
  `name` VARCHAR(50) DEFAULT NULL COMMENT '姓名',
  `dept_id` BIGINT DEFAULT NULL COMMENT '部门id',
  `number_of_people` BIGINT DEFAULT '0' COMMENT '人数',
  `folder_id` BIGINT DEFAULT NULL COMMENT '附件',
  `destination` VARCHAR(100) DEFAULT NULL COMMENT '目的地',
  `liaison_man` VARCHAR(50) DEFAULT NULL COMMENT '联系人',
  `phone` VARCHAR(50) DEFAULT NULL COMMENT '联系电话',
  `set_out_date` DATETIME DEFAULT NULL COMMENT '出发时间',
  `leader_id` BIGINT DEFAULT NULL COMMENT '分管领导id',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=INNODB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_0900_ai_ci COMMENT='公务派车';


-- -----------------------------------------------------------
-- 菜单部门关联表
-- -----------------------------------------------------------
DROP TABLE IF EXISTS xjr_menu_dept_relation;
CREATE TABLE `xjr_menu_dept_relation` (
  `id` BIGINT NOT NULL,
  `create_user_id` BIGINT DEFAULT NULL,
  `create_date` DATETIME(3) DEFAULT NULL,
  `modify_user_id` BIGINT DEFAULT NULL,
  `modify_date` DATETIME(3) DEFAULT NULL,
  `delete_mark` INT DEFAULT NULL,
  `enabled_mark` INT DEFAULT NULL,
  `menu_id` BIGINT DEFAULT NULL COMMENT '菜单id',
  `dept_id` BIGINT DEFAULT NULL COMMENT '部门id',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=INNODB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_0900_ai_ci COMMENT='菜单部门关联表';

ALTER TABLE `pb_v_xsxxsfytb`   
  ADD INDEX (`personalid`),
  ADD INDEX (`feeitemcode`);
  
ALTER TABLE `pb_v_xssfdetail`   
  ADD INDEX (`personalid`),
  ADD INDEX (`feeitemcode`);
  
  ALTER TABLE `base_student_school_roll`   
  ADD INDEX (`student_type`),
  ADD INDEX (`archives_status`),
  ADD INDEX (`stduy_status`);

ALTER TABLE `base_student_family_member`   
  ADD INDEX (`user_id`);

ALTER TABLE `pb_v_xsxxsfytb`   
ADD INDEX (`Studentcode`);

ALTER TABLE `pb_v_xsxxsfytb`   
  ADD INDEX (`jfzt`),
  ADD INDEX (`beltcode`);

-- -----------------------------------------------------------
-- 公务派车表增加出发时间
-- -----------------------------------------------------------
DROP PROCEDURE IF EXISTS createCom;
DELIMITER $$
CREATE
    PROCEDURE createCom()
BEGIN
    IF NOT EXISTS(SELECT 1 FROM information_schema.columns  WHERE table_name='wf_official_vehicles' AND COLUMN_NAME = 'set_out_date') THEN
ALTER TABLE `wf_official_vehicles`   
  ADD COLUMN `set_out_date` DATETIME NULL   COMMENT '出发时间' AFTER `phone`;

    END IF;
END$$
DELIMITER ;
CALL createCom;
DROP PROCEDURE createCom;

ALTER TABLE `xjr_workflow_record`   
  ADD COLUMN `show_node` BOOLEAN NULL   COMMENT '是否显示该记录' AFTER `create_user_id`;
ALTER TABLE `xjr_workflow_operate_record`   
  ADD COLUMN `show_node` BOOLEAN NULL   COMMENT '是否显示该记录';
  ALTER TABLE `xjr_workflow_approve_record`   
  ADD COLUMN `show_node` BOOLEAN NULL   COMMENT '是否显示该记录';

DROP TABLE IF EXISTS wf_document_circulation;
CREATE TABLE `wf_document_circulation` (
  `id` BIGINT NOT NULL COMMENT '主键编号',
  `create_user_id` BIGINT DEFAULT NULL COMMENT '创建人',
  `create_date` DATETIME DEFAULT NULL COMMENT '创建时间',
  `modify_user_id` BIGINT DEFAULT NULL COMMENT '修改人',
  `modify_date` DATETIME DEFAULT NULL COMMENT '修改时间',
  `delete_mark` INT DEFAULT NULL COMMENT '删除标志',
  `enabled_mark` INT DEFAULT NULL COMMENT '有效标志',
  `file_title` VARCHAR(255) CHARACTER SET utf8mb4 COLLATE utf8mb4_0900_ai_ci DEFAULT NULL COMMENT '文件标题',
  `dispatch_unit` VARCHAR(255) CHARACTER SET utf8mb4 COLLATE utf8mb4_0900_ai_ci DEFAULT NULL COMMENT '发文单位',
  `Issue_number` VARCHAR(200) CHARACTER SET utf8mb4 COLLATE utf8mb4_0900_ai_ci DEFAULT NULL COMMENT '期号',
  `date` DATETIME DEFAULT NULL COMMENT '日期',
  `file_id` BIGINT DEFAULT NULL COMMENT '附件文件id',
  `status` INT DEFAULT NULL COMMENT '流程状态（0:未结束 1：结束）',
  PRIMARY KEY (`id`)
) ENGINE=INNODB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_0900_ai_ci COMMENT='文件传阅';


INSERT INTO `tl`.`xjr_form_template` (`id`, `name`, `category`, `form_type`, `form_json`, `sort_code`, `remark`, `create_user_id`, `create_date`, `modify_user_id`, `modify_date`, `delete_mark`, `enabled_mark`, `config_json`, `form_design_type`) VALUES (1778625118535204865, '公务派车', 1650387621322219521, 1, '{\"tableConfigs\":[{\"pkField\":\"id\",\"isMain\":true,\"relationField\":null,\"tableName\":\"wf_official_vehicles\",\"relationTableField\":null,\"pkType\":\"Long\"}],\"dataAuthList\":[],\"tableStructureConfigs\":[{\"pkField\":null,\"isMain\":true,\"tableComment\":null,\"operator\":4,\"tableName\":\"wf_official_vehicles\",\"tableFieldConfigs\":[{\"fieldName\":\"name\",\"fieldComment\":\"申请人\",\"fieldType\":null,\"fieldLength\":50},{\"fieldName\":\"dept_id\",\"fieldComment\":\"所属部门\",\"fieldType\":null,\"fieldLength\":50},{\"fieldName\":\"destination\",\"fieldComment\":\"目的地\",\"fieldType\":null,\"fieldLength\":50},{\"fieldName\":\"set_out_date\",\"fieldComment\":\"出发时间\",\"fieldType\":null,\"fieldLength\":50},{\"fieldName\":\"number_of_people\",\"fieldComment\":\"出行人数\",\"fieldType\":null,\"fieldLength\":50},{\"fieldName\":\"leader_id\",\"fieldComment\":\"领导id\",\"fieldType\":null,\"fieldLength\":50},{\"fieldName\":\"liaison_man\",\"fieldComment\":\"联系人\",\"fieldType\":null,\"fieldLength\":50},{\"fieldName\":\"phone\",\"fieldComment\":\"联系电话\",\"fieldType\":null,\"fieldLength\":50},{\"fieldName\":\"folder_id\",\"fieldComment\":\"附件\",\"fieldType\":null,\"fieldLength\":50},{\"fieldName\":\"status\",\"fieldComment\":\"status\",\"fieldType\":null,\"fieldLength\":50}]}],\"isDataAuth\":false,\"formEventConfig\":{\"0\":[{\"color\":\"#2774ff\",\"icon\":\"#icon-kaishi\",\"type\":\"circle\",\"bgcColor\":\"#D8E5FF\",\"isClick\":false,\"isUserDefined\":false,\"text\":\"开始节点\"},{\"color\":\"#F6AB01\",\"icon\":\"#icon-chushihua\",\"bgcColor\":\"#f9f5ea\",\"isClick\":false,\"isUserDefined\":false,\"text\":\"初始化表单\",\"nodeInfo\":{\"processEvent\":[]}},{\"color\":\"#2774FF\",\"icon\":\"#icon-yonghu-xianxing\",\"bgcColor\":\"#F5F8FA\",\"isClick\":true,\"isUserDefined\":true,\"text\":\"用户自定义节点\",\"nodeInfo\":{\"processEvent\":[{\"operateConfig\":\"if(formModel.folder_id == \\\"\\\"||formModel.folder_id == null)\\n{\\nformModel.folder_id = 0;\\n}\",\"operateType\":\"js\",\"showValue\":\"已配置\"}]}}],\"1\":[{\"color\":\"#B36EDB\",\"icon\":\"#icon-shujufenxi\",\"bgcColor\":\"#F8F2FC\",\"isClick\":false,\"isUserDefined\":false,\"text\":\"获取表单数据\",\"detail\":\"(新增无此操作)\",\"nodeInfo\":{\"processEvent\":[]}}],\"2\":[{\"color\":\"#F8625C\",\"icon\":\"#icon-jiazai\",\"bgcColor\":\"#FFF1F1\",\"isClick\":false,\"isUserDefined\":false,\"text\":\"加载表单\",\"nodeInfo\":{\"processEvent\":[]}}],\"3\":[{\"color\":\"#6C6AE0\",\"icon\":\"#icon-jsontijiao\",\"bgcColor\":\"#F5F4FF\",\"isClick\":false,\"isUserDefined\":false,\"text\":\"提交表单\",\"nodeInfo\":{\"processEvent\":[]}}],\"4\":[{\"color\":\"#F8625C\",\"isLast\":true,\"icon\":\"#icon-jieshuzhiliao\",\"type\":\"circle\",\"bgcColor\":\"#FFD6D6\",\"isClick\":false,\"isUserDefined\":false,\"text\":\"结束节点\"}]},\"formJson\":{\"hiddenComponent\":[{\"code\":\"status\",\"label\":\"status\",\"type\":\"hiddenComponent\",\"bindField\":\"status\",\"bindTable\":\"wf_official_vehicles\",\"value\":\"0\",\"key\":\"hiddenComponent1712892228526\"}],\"list\":[{\"code\":null,\"rules\":[],\"label\":\"栅格布局\",\"type\":\"grid\",\"isSubFormChild\":false,\"layout\":[{\"list\":[{\"code\":null,\"rules\":[],\"label\":\"申请人\",\"type\":\"info\",\"isSubFormChild\":false,\"layout\":null,\"bindStartTime\":null,\"children\":null,\"bindField\":\"name\",\"options\":{\"showLabel\":true,\"isShow\":true,\"infoType\":0,\"width\":\"100%\",\"disabled\":true,\"placeholder\":\"\",\"loadAgain\":false,\"events\":{},\"span\":7},\"bindTable\":\"wf_official_vehicles\",\"value\":null,\"key\":\"1cb60b1274d54bac9bd0774980526f88\",\"bindEndTime\":null}],\"name\":null,\"span\":12},{\"list\":[{\"code\":null,\"rules\":null,\"label\":\"所属部门\",\"type\":\"select\",\"isSubFormChild\":false,\"layout\":null,\"bindStartTime\":null,\"children\":null,\"bindField\":\"dept_id\",\"options\":{\"clearable\":false,\"multiple\":\"combobox\",\"defaultSelect\":null,\"rules\":[],\"params\":null,\"valueField\":\"value\",\"showLabel\":true,\"required\":true,\"labelField\":\"label\",\"isShow\":true,\"datasourceType\":\"api\",\"dicOptions\":[],\"showSearch\":true,\"width\":\"100%\",\"apiConfig\":{\"method\":\"GET\",\"script\":\"return db.select(\\\"\\\"\\\"\\r\\n\\tSELECT t.name as label,t.id AS value FROM xjr_department  t\\r\\n\\r\\nWHERE t.parent_id = 1684107782861680000 AND t.delete_mark = 0 AND t.code !=10001\\r\\n\\\"\\\"\\\");\",\"apiParams\":[{\"tableInfo\":[],\"title\":\"Query Params\",\"key\":\"1\"},{\"tableInfo\":[],\"title\":\"Header\",\"key\":\"2\"},{\"title\":\"Body\",\"key\":\"3\"}],\"path\":\"baseData/department/option\",\"apiId\":\"090dc643d95b4efc92b1c44d41fdde86\"},\"disabled\":false,\"placeholder\":\"请选择所属部门\",\"staticOptions\":[{\"label\":\"Option 1\",\"value\":\"Option 1\",\"key\":1},{\"label\":\"Option 2\",\"value\":\"Option 2\",\"key\":2},{\"label\":\"Option 3\",\"value\":\"Option 3\",\"key\":3}],\"events\":{\"change\":\"var isPc = false;\\ntry{\\nisPc = window.isPc;\\n}\\ncatch(_){};\\nif(formModel.isChange || !isPc ){\\nformActionType.httpRequest({\\n    requestType: \'get\', //请求方式有: get、post、put、delete\\n    requestUrl: \'/magic-api/educational/depart-leader\', //请求地址\\n    params: {\\n      dep_id:formModel.dept_id\\n    },//请求参数\\n    errorMessageMode: \'none\' //错误提示方式，默认为none\\n}).then(function(res){//行动\\nvar v=res.data||res;\\n formModel.leader_id=v;\\n})\\n};\",\"click\":\"formModel.isChange = true;\"},\"span\":7},\"bindTable\":\"wf_official_vehicles\",\"value\":null,\"key\":\"4fd6ee2489164c329ca99d54c66c3f8f\",\"bindEndTime\":null}],\"name\":null,\"span\":12}],\"bindStartTime\":null,\"children\":null,\"bindField\":null,\"options\":{\"align\":\"top\",\"isShow\":true,\"gutter\":16,\"justify\":\"start\"},\"bindTable\":null,\"value\":null,\"key\":\"bf6e564972b644e4be49f039daa28ec3\",\"bindEndTime\":null},{\"code\":null,\"rules\":[],\"label\":\"栅格布局\",\"type\":\"grid\",\"isSubFormChild\":false,\"layout\":[{\"list\":[{\"code\":null,\"rules\":null,\"label\":\"目的地\",\"type\":\"input\",\"isSubFormChild\":false,\"layout\":null,\"bindStartTime\":null,\"children\":null,\"bindField\":\"destination\",\"options\":{\"maxlength\":null,\"defaultValue\":\"\",\"prefix\":\"\",\"rules\":[],\"suffix\":\"\",\"addonAfter\":\"\",\"showLabel\":true,\"required\":true,\"isSave\":false,\"isShow\":true,\"width\":\"100%\",\"disabled\":false,\"placeholder\":\"请输入目的地\",\"allowClear\":false,\"addonBefore\":\"\",\"events\":{},\"span\":7},\"bindTable\":\"wf_official_vehicles\",\"value\":null,\"key\":\"e8d25ac9f8b34be988e216f274c14fef\",\"bindEndTime\":null},{\"code\":null,\"rules\":null,\"label\":\"出发时间\",\"type\":\"date\",\"isSubFormChild\":false,\"layout\":null,\"bindStartTime\":null,\"children\":null,\"bindField\":\"set_out_date\",\"options\":{\"defaultValue\":\"\",\"format\":\"YYYY-MM-DD HH:mm:ss\",\"rules\":[],\"showLabel\":true,\"required\":false,\"isShow\":true,\"width\":\"100%\",\"disabled\":false,\"placeholder\":\"请选择出发时间\",\"allowClear\":true,\"events\":{},\"span\":7},\"bindTable\":\"wf_official_vehicles\",\"value\":null,\"key\":\"16413226567f4b5581cf6ea5825e930a\",\"bindEndTime\":null}],\"name\":null,\"span\":12},{\"list\":[{\"code\":null,\"rules\":null,\"label\":\"出行人数\",\"type\":\"number\",\"isSubFormChild\":false,\"layout\":null,\"bindStartTime\":null,\"children\":null,\"bindField\":\"number_of_people\",\"options\":{\"controls\":true,\"max\":100,\"maxlength\":null,\"defaultValue\":null,\"rules\":[],\"subTotal\":false,\"showLabel\":true,\"required\":true,\"isShow\":true,\"min\":0,\"width\":\"100%\",\"step\":1,\"disabled\":false,\"events\":{},\"span\":7},\"bindTable\":\"wf_official_vehicles\",\"value\":null,\"key\":\"1fc2db6441184964997dd1f5d9ef5102\",\"bindEndTime\":null},{\"code\":null,\"rules\":[],\"label\":\"领导id\",\"type\":\"input\",\"isSubFormChild\":false,\"layout\":null,\"bindStartTime\":null,\"children\":null,\"bindField\":\"leader_id\",\"options\":{\"maxlength\":null,\"defaultValue\":\"0\",\"prefix\":\"\",\"rules\":[],\"suffix\":\"\",\"addonAfter\":\"\",\"showLabel\":true,\"required\":false,\"isSave\":false,\"isShow\":true,\"width\":\"100%\",\"disabled\":true,\"placeholder\":\"请输入领导id\",\"allowClear\":false,\"addonBefore\":\"\",\"events\":{},\"span\":7},\"bindTable\":\"wf_official_vehicles\",\"value\":null,\"key\":\"d0a61379637743a7aa62e6b69ba8a58e\",\"bindEndTime\":null}],\"name\":null,\"span\":12}],\"bindStartTime\":null,\"children\":null,\"bindField\":null,\"options\":{\"align\":\"top\",\"isShow\":true,\"gutter\":16,\"justify\":\"start\"},\"bindTable\":null,\"value\":null,\"key\":\"4a1518041fe449a3b54da4b574dff919\",\"bindEndTime\":null},{\"code\":null,\"rules\":[],\"label\":\"栅格布局\",\"type\":\"grid\",\"isSubFormChild\":false,\"layout\":[{\"list\":[{\"code\":null,\"rules\":null,\"label\":\"联系人\",\"type\":\"input\",\"isSubFormChild\":false,\"layout\":null,\"bindStartTime\":null,\"children\":null,\"bindField\":\"liaison_man\",\"options\":{\"maxlength\":null,\"defaultValue\":\"\",\"prefix\":\"\",\"rules\":[],\"suffix\":\"\",\"addonAfter\":\"\",\"showLabel\":true,\"required\":true,\"isSave\":false,\"isShow\":true,\"width\":\"100%\",\"disabled\":false,\"placeholder\":\"请输入联系人\",\"allowClear\":false,\"addonBefore\":\"\",\"events\":{},\"span\":7},\"bindTable\":\"wf_official_vehicles\",\"value\":null,\"key\":\"01488459443c4fe6b138e23b808f03ad\",\"bindEndTime\":null}],\"name\":null,\"span\":12},{\"list\":[{\"code\":null,\"rules\":null,\"label\":\"联系电话\",\"type\":\"input\",\"isSubFormChild\":false,\"layout\":null,\"bindStartTime\":null,\"children\":null,\"bindField\":\"phone\",\"options\":{\"maxlength\":null,\"defaultValue\":\"\",\"prefix\":\"\",\"rules\":[],\"suffix\":\"\",\"addonAfter\":\"\",\"showLabel\":true,\"required\":true,\"isSave\":false,\"isShow\":true,\"width\":\"100%\",\"disabled\":false,\"placeholder\":\"请输入联系电话\",\"allowClear\":false,\"addonBefore\":\"\",\"events\":{},\"span\":7},\"bindTable\":\"wf_official_vehicles\",\"value\":null,\"key\":\"40e5e5462b674895a9882fe7adc423d2\",\"bindEndTime\":null}],\"name\":null,\"span\":12}],\"bindStartTime\":null,\"children\":null,\"bindField\":null,\"options\":{\"align\":\"top\",\"isShow\":true,\"gutter\":16,\"justify\":\"start\"},\"bindTable\":null,\"value\":null,\"key\":\"3a8b5f27f1b641d287146f8bb847b733\",\"bindEndTime\":null},{\"code\":null,\"rules\":[],\"label\":\"附件\",\"type\":\"upload\",\"isSubFormChild\":false,\"layout\":null,\"bindStartTime\":null,\"children\":null,\"bindField\":\"folder_id\",\"options\":{\"defaultValue\":[],\"multiple\":false,\"maxSize\":5,\"listType\":\"text\",\"showLabel\":true,\"required\":false,\"accept\":\"\",\"isShow\":true,\"maxNumber\":5,\"disabled\":false,\"events\":{},\"span\":\"\"},\"bindTable\":\"wf_official_vehicles\",\"value\":null,\"key\":\"c840664d2fc14a859bbadc46d0349b29\",\"bindEndTime\":null},{\"code\":\"status\",\"rules\":null,\"label\":\"status\",\"type\":\"hiddenComponent\",\"isSubFormChild\":null,\"layout\":null,\"bindStartTime\":null,\"children\":null,\"bindField\":\"status\",\"options\":null,\"bindTable\":\"wf_official_vehicles\",\"value\":\"0\",\"key\":\"hiddenComponent1712892228526\",\"bindEndTime\":null}],\"config\":{\"formType\":\"modal\",\"hideRequiredMark\":null,\"formWidth\":800,\"layout\":\"vertical\",\"size\":\"default\",\"labelAlign\":\"right\",\"labelCol\":{\"offset\":0,\"span\":3}}},\"databaseId\":\"master\"}', NULL, '', 1000000000000000000, '2024-04-12 11:24:08.173', 1000000000000000000, '2024-04-12 15:45:52.409', 0, 1, NULL, 0);
INSERT INTO `tl`.`xjr_form_template` (`id`, `name`, `category`, `form_type`, `form_json`, `sort_code`, `remark`, `create_user_id`, `create_date`, `modify_user_id`, `modify_date`, `delete_mark`, `enabled_mark`, `config_json`, `form_design_type`) VALUES (1778676770646573058, '文件传阅', 1650387621322219521, 1, '{\"tableConfigs\":[{\"pkField\":\"id\",\"isMain\":true,\"relationField\":null,\"tableName\":\"wf_document_circulation\",\"relationTableField\":null,\"pkType\":\"Long\"}],\"dataAuthList\":[],\"tableStructureConfigs\":[{\"pkField\":null,\"isMain\":true,\"tableComment\":null,\"operator\":4,\"tableName\":\"wf_document_circulation\",\"tableFieldConfigs\":[{\"fieldName\":\"file_title\",\"fieldComment\":\"文件标题\",\"fieldType\":null,\"fieldLength\":50},{\"fieldName\":\"dispatch_unit\",\"fieldComment\":\"发文单位\",\"fieldType\":null,\"fieldLength\":50},{\"fieldName\":\"Issue_number\",\"fieldComment\":\"期号\",\"fieldType\":null,\"fieldLength\":50},{\"fieldName\":\"date\",\"fieldComment\":\"日期\",\"fieldType\":null,\"fieldLength\":50},{\"fieldName\":\"file_id\",\"fieldComment\":\"附件\",\"fieldType\":null,\"fieldLength\":50},{\"fieldName\":\"status\",\"fieldComment\":\"流程状态\",\"fieldType\":null,\"fieldLength\":50}]}],\"isDataAuth\":false,\"formEventConfig\":{\"0\":[{\"color\":\"#2774ff\",\"icon\":\"#icon-kaishi\",\"type\":\"circle\",\"bgcColor\":\"#D8E5FF\",\"isUserDefined\":false,\"text\":\"开始节点\"},{\"color\":\"#F6AB01\",\"icon\":\"#icon-chushihua\",\"bgcColor\":\"#f9f5ea\",\"isUserDefined\":false,\"text\":\"初始化表单\",\"nodeInfo\":{\"processEvent\":[]}}],\"1\":[{\"color\":\"#B36EDB\",\"icon\":\"#icon-shujufenxi\",\"bgcColor\":\"#F8F2FC\",\"isUserDefined\":false,\"text\":\"获取表单数据\",\"detail\":\"(新增无此操作)\",\"nodeInfo\":{\"processEvent\":[]}}],\"2\":[{\"color\":\"#F8625C\",\"icon\":\"#icon-jiazai\",\"bgcColor\":\"#FFF1F1\",\"isUserDefined\":false,\"text\":\"加载表单\",\"nodeInfo\":{\"processEvent\":[]}}],\"3\":[{\"color\":\"#6C6AE0\",\"icon\":\"#icon-jsontijiao\",\"bgcColor\":\"#F5F4FF\",\"isUserDefined\":false,\"text\":\"提交表单\",\"nodeInfo\":{\"processEvent\":[]}}],\"4\":[{\"color\":\"#F8625C\",\"isLast\":true,\"icon\":\"#icon-jieshuzhiliao\",\"type\":\"circle\",\"bgcColor\":\"#FFD6D6\",\"isUserDefined\":false,\"text\":\"结束节点\"}]},\"formJson\":{\"hiddenComponent\":[{\"code\":\"status\",\"label\":\"流程状态\",\"type\":\"hiddenComponent\",\"bindField\":\"status\",\"bindTable\":\"wf_document_circulation\",\"value\":\"0\",\"key\":\"hiddenComponent1712904534382\"}],\"list\":[{\"code\":null,\"rules\":[],\"label\":\"文件标题\",\"type\":\"input\",\"isSubFormChild\":false,\"layout\":null,\"bindStartTime\":null,\"children\":null,\"bindField\":\"file_title\",\"options\":{\"maxlength\":null,\"defaultValue\":\"\",\"prefix\":\"\",\"rules\":[],\"suffix\":\"\",\"addonAfter\":\"\",\"showLabel\":true,\"required\":true,\"isSave\":false,\"isShow\":true,\"width\":\"100%\",\"disabled\":false,\"placeholder\":\"请输入文件标题\",\"allowClear\":false,\"addonBefore\":\"\",\"events\":{},\"span\":\"\"},\"bindTable\":\"wf_document_circulation\",\"value\":null,\"key\":\"4a26042339994f339ef9d1c9011ad758\",\"bindEndTime\":null},{\"code\":null,\"rules\":[],\"label\":\"发文单位\",\"type\":\"input\",\"isSubFormChild\":false,\"layout\":null,\"bindStartTime\":null,\"children\":null,\"bindField\":\"dispatch_unit\",\"options\":{\"maxlength\":null,\"defaultValue\":\"\",\"prefix\":\"\",\"rules\":[],\"suffix\":\"\",\"addonAfter\":\"\",\"showLabel\":true,\"required\":false,\"isSave\":false,\"isShow\":true,\"width\":\"100%\",\"disabled\":false,\"placeholder\":\"请输入发文单位\",\"allowClear\":false,\"addonBefore\":\"\",\"events\":{},\"span\":\"\"},\"bindTable\":\"wf_document_circulation\",\"value\":null,\"key\":\"1e984e2f2d2941ca9da86bd1a662b185\",\"bindEndTime\":null},{\"code\":null,\"rules\":[],\"label\":\"栅格布局\",\"type\":\"grid\",\"isSubFormChild\":false,\"layout\":[{\"list\":[{\"code\":null,\"rules\":null,\"label\":\"期号\",\"type\":\"input\",\"isSubFormChild\":false,\"layout\":null,\"bindStartTime\":null,\"children\":null,\"bindField\":\"Issue_number\",\"options\":{\"maxlength\":null,\"defaultValue\":\"\",\"prefix\":\"\",\"rules\":[],\"suffix\":\"\",\"addonAfter\":\"\",\"showLabel\":true,\"required\":false,\"isSave\":false,\"isShow\":true,\"width\":\"100%\",\"disabled\":false,\"placeholder\":\"请输入期号\",\"allowClear\":false,\"addonBefore\":\"\",\"events\":{},\"span\":7},\"bindTable\":\"wf_document_circulation\",\"value\":null,\"key\":\"432dabdaf575429bb374e1c4bbe57ff2\",\"bindEndTime\":null}],\"name\":null,\"span\":12},{\"list\":[{\"code\":null,\"rules\":null,\"label\":\"日期\",\"type\":\"date\",\"isSubFormChild\":false,\"layout\":null,\"bindStartTime\":null,\"children\":null,\"bindField\":\"date\",\"options\":{\"defaultValue\":\"\",\"format\":\"YYYY-MM-DD\",\"rules\":[],\"showLabel\":true,\"required\":false,\"isShow\":true,\"width\":\"100%\",\"disabled\":false,\"placeholder\":\"请选择日期\",\"allowClear\":true,\"events\":{},\"span\":7},\"bindTable\":\"wf_document_circulation\",\"value\":null,\"key\":\"60a768b3324c4e7d999d7f61cd4eaaef\",\"bindEndTime\":null}],\"name\":null,\"span\":12}],\"bindStartTime\":null,\"children\":null,\"bindField\":null,\"options\":{\"align\":\"top\",\"isShow\":true,\"gutter\":16,\"justify\":\"start\"},\"bindTable\":null,\"value\":null,\"key\":\"23d93d2c1fbb45dc9a2a163bd27ba321\",\"bindEndTime\":null},{\"code\":null,\"rules\":[],\"label\":\"附件\",\"type\":\"upload\",\"isSubFormChild\":false,\"layout\":null,\"bindStartTime\":null,\"children\":null,\"bindField\":\"file_id\",\"options\":{\"defaultValue\":[],\"multiple\":false,\"maxSize\":50,\"listType\":\"text\",\"showLabel\":true,\"required\":true,\"accept\":\"\",\"isShow\":true,\"maxNumber\":10,\"disabled\":false,\"events\":{},\"span\":\"\"},\"bindTable\":\"wf_document_circulation\",\"value\":null,\"key\":\"2ff6cc27a65144c68d507f554a50b39a\",\"bindEndTime\":null},{\"code\":\"status\",\"rules\":null,\"label\":\"流程状态\",\"type\":\"hiddenComponent\",\"isSubFormChild\":null,\"layout\":null,\"bindStartTime\":null,\"children\":null,\"bindField\":\"status\",\"options\":null,\"bindTable\":\"wf_document_circulation\",\"value\":\"0\",\"key\":\"hiddenComponent1712904534382\",\"bindEndTime\":null}],\"config\":{\"formType\":\"modal\",\"hideRequiredMark\":null,\"formWidth\":800,\"layout\":\"vertical\",\"size\":\"default\",\"labelAlign\":\"right\",\"labelCol\":{\"offset\":0,\"span\":3}}},\"databaseId\":\"master\"}', NULL, '文件传阅', 1000000000000000000, '2024-04-12 14:49:23.000', NULL, NULL, 0, 1, NULL, 0);

